//
//  WorkoutManager.swift
//  testiOS
//
//  Created by Easten Wember on 3/29/22.
//

import Foundation
import HealthKit //want healthKit APIs!!
import OSLog
import UserNotifications

class WorkoutManager: NSObject, ObservableObject {
    
    let healthStore = HKHealthStore()

    // request authorization to access HealthKit
    func requestAuthorization() {
        //quantity type to write to the health store
        let typesToShare: Set = [
            HKQuantityType.workoutType()
        ]

        // The quantity types to read from the health store.
        let typesToRead: Set = [
            
            //.quantityType(forIdentifier: .respiratoryRate)!,
            HKQuantityType.quantityType(forIdentifier: .heartRate)!,
            //HKQuantityType.quantityType(forIdentifier: .appleStandTime)!,
            HKQuantityType.quantityType(forIdentifier: .restingHeartRate)!,
            HKSampleType.quantityType(forIdentifier: .restingHeartRate)!
            
            //HKObjectType.activitySummaryType() //want permission to read the activity rings summary
        ]

        //request authorization for those quantity types
        healthStore.requestAuthorization(toShare: typesToShare, read: typesToRead) { (success, error) in
            //handle error
            print("here in healthStore request authorization")
        }
    }

   

    //workout metrics
    @Published var averageHeartRate: Double = 0
    @Published var restingHeartRate: Double = 0
    @Published var heartRate: Double = 0
    
    
    //*may need to add here ? to calculate some ..use to display in metricsView OR to push notifications
    //HKQauntityTypeIdentifiers...not sure what their types are??
    @Published var respRate : Double = 0
    @Published var restHR : Double = 0 //comp to current HR/AVG
    @Published var standTime : Int = 0
    
    //var to control rate of notification...for now
    @Published var countHRNotifs : Int = 20
    
    
    func getRespRate(_ statistics: HKStatistics?){ //work on this...see if can grab a diff way?? HKUnit is wrong!! keeps causing error
        guard let statistics = statistics else { return } //unwraps statisitcs
        
        let respRate = HKUnit.count() //count be .count? thouight like Beat per minute
        self.respRate = statistics.averageQuantity()?.doubleValue(for: respRate) ?? 0
        print("in func resp rate: \(self.respRate)")
    }
    
    

    func updateForStatistics(_ statistics: HKStatistics?) { //not called...used in watch workout manager
        
        guard let statistics = statistics else { return }

        DispatchQueue.main.async {
            //print("hello")
            switch statistics.quantityType {
                //get HR#imageLiteral(resourceName: "04122001.heic")
            case HKQuantityType.quantityType(forIdentifier: .heartRate):
                let heartRateUnit = HKUnit.count().unitDivided(by: HKUnit.minute())
                self.heartRate = statistics.mostRecentQuantity()?.doubleValue(for: heartRateUnit) ?? 0
                self.averageHeartRate = statistics.averageQuantity()?.doubleValue(for: heartRateUnit) ?? 0
                print("curr HR: \(self.heartRate)")
                //self.heartRateNotification()
                //self.getStepCount(statistics) //attempt to grab stepLength here
                //self.getRespRate(statistics) //call respRate function
                
                
            //attempt to get respiaroy rate
//            case HKQuantityType.quantityType(forIdentifier: .respiratoryRate):
//                let respRate = HKUnit.minute() //count be .count? thouight like Beat per minute
//                self.respRate = statistics.averageQuantity()?.doubleValue(for: respRate) ?? 0
//                print("in case resp rate: \(self.respRate)")
                
                

            default:
                return
            }
        } //end switch
    }


    func heartRateNotification() { //left off here..enters but no notification show on watch...
        print("in heartRateNotification..count: \(countHRNotifs)")

        getRestHR()
        
        if countHRNotifs == 20{
            print("should send notification to user...")
            let center = UNUserNotificationCenter.current()

            let content = UNMutableNotificationContent()
            content.title = "Stress Update"
            content.subtitle = "Resting HR is: \(self.restingHeartRate)"
            //content.subtitle = "Avg HR is: \(self.averageHeartRate)"

            
            if theHR > restingHeartRate{
                content.body = "current HR \(self.heartRate) above resting!"
            }
            else{
                content.body = "current HR \(self.heartRate) below resting!"
            }
            
        
            content.categoryIdentifier = "alarm"
            content.userInfo = ["customData": "fizzbuzz"]
            content.sound = UNNotificationSound.default

            //deliver notification after specified time interval...60 seconds * 1 for one minute..
            //notification only goes once ...repeats:false
            let trigger = UNTimeIntervalNotificationTrigger(timeInterval: (1*60), repeats: false)

            let request = UNNotificationRequest(identifier: UUID().uuidString, content: content, trigger: trigger)
            center.add(request)
            
            //**EVENTUALLY SOLVE THIS WITH TIME INTERVAL/REPEATS on timeintervalnotification trigger
            countHRNotifs = 0 //reset count to space out notifications user gets
        }
        else{
            countHRNotifs += 1 //work way back up to 20
        }
        
    }//end heartRateNotification
    

//vars for getLatestHR fcn
    @Published var theHR : Double = 0.0 //published var so LandingView can access value
    @Published var timeDate : String = ""
    var count : Int = 0 //see how many times we run HR fcn
    
    @Published var theDateString : String = "N/A"
    @Published var theRestingString : String = "N/A"
    
    func getLatestHR() {
        count+=1 // see how many times we're trying to update HR
        //want to grab hr sample
        guard let sampleType = HKObjectType.quantityType(forIdentifier: .heartRate) else{
            return
        }
        
        let startDate = Calendar.current.date(byAdding: .month, value: -1, to: Date())
        

        //let endDate = Date()
        
        let predicate = HKQuery.predicateForSamples(withStart: startDate, end: Date(), options: HKQueryOptions.strictEndDate)
        
        let sortDescriptor = NSSortDescriptor(key: HKSampleSortIdentifierStartDate, ascending: false)
        
        //result handler to health store executuin query
        let query = HKSampleQuery(sampleType: sampleType, predicate: predicate, limit: Int(HKObjectQueryNoLimit), sortDescriptors: [sortDescriptor]) { (sample, result, error) in
            guard error == nil else{
                print("query issue")
                return
            }//end error guard
            
            let data = result![0] as! HKQuantitySample //grab the data in pos 0..should be HKQuantSamp HR
            let unit = HKUnit(from: "count/min")
            let latestHR = data.quantity.doubleValue(for: unit)
            print("Latest HR: \(latestHR) BPM")
            print("count: \(self.count)")
            
            self.theHR = latestHR //give theHR whatever updated HR is...put in theHR bc we know the type of var for sure
            
            //used to be let(s)..tried as vars bc let = immutable..need var instead of let! else doesnt update?
            let dateFormator = DateFormatter()
            dateFormator.dateFormat = "hh:mm:ss dd/MM/yyyy"
            let StartDate = dateFormator.string(from: data.startDate)
            let EndDate = dateFormator.string(from: data.endDate)
            print("StartDate: \(StartDate) ... EndDate: \(EndDate)")
            
            let timeFormator = DateFormatter() //used to print just time next to previous heart rates
            timeFormator.dateFormat = "hh:mm:ss"
            self.timeDate = timeFormator.string(from: data.startDate) //hold time of queries
            print("Time date: \(self.timeDate)")
            
            self.theDateString = StartDate //give theDateString the start date of curr query
            
        }//end query
        
        healthStore.execute(query)
        
        updateHRarr() //update hrarr with curr HR
    
    }//end func get latestHR
    
    @Published var hrarr = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    @Published var hrtimes = ["", "", "", "", "", "", "", "", "", ""] //hold time of hr queries
    @Published var lastHR:  Double = 0.0 //compare to whatever last HR was
    @Published var index : Int = 0 //start at arr index 0
    @Published var theDate: Date = Date()
    @Published var dateF: DateFormatter = DateFormatter()
    
   

    func updateHRarr(){
        if theHR != lastHR{ //if the current HR is != the last HR..is a new vaLue
//        if hrarr[0] != theHR{ //only want new values
            
            var size = index; //use to shift only for as many values in the arr
            for i in 0...size{
                print("i: \(i)")
                if (size - 1) != -1 { //dont want index to negative, we'll update [0] at end
                    print("inside for...size: \(size)")

                    hrarr[size] = hrarr[size-1]; //grab behind it
                    hrtimes[size] = hrtimes[size-1] //grab time behind it
                    size = size - 1;
                }
            }//end for shifting items down
                
            hrarr[0] = theHR; //new value goes in [0]..on top
            hrtimes[0] = timeDate;
            index += 1;
            print("index: \(index)")
            if index == 10{ //only 10 spaces in arr, so stop...might need this to be at top
                return;
            }
            
            lastHR = theHR; //mark the HR we're on...only want "new"/updated values to be shown in array
            
          
        } //end if
        
    }//end func updateHRarr
    
    
    //use to create notifications?...query for resting heart rate done here!
    func getRestHR(){
        
        guard let sampleType = HKObjectType.quantityType(forIdentifier: .restingHeartRate) else{
            return
        }
        
        let startDate = Calendar.current.date(byAdding: .month, value: -1, to: Date())
                
        let predicate = HKQuery.predicateForSamples(withStart: startDate, end: Date(), options: HKQueryOptions.strictEndDate)
        
        let sortDescriptor = NSSortDescriptor(key: HKSampleSortIdentifierStartDate, ascending: false)
        
        //result handler to health store executuin query
        let query = HKSampleQuery(sampleType: sampleType, predicate: predicate, limit: Int(HKObjectQueryNoLimit), sortDescriptors: [sortDescriptor]) { (sample, result, error) in
            guard error == nil else{
                print("query issue")
                return
            }//end error guard
            
            let data = result![0] as! HKQuantitySample //grab the data in pos 0..should be HKQuantSamp restingHR
            let unit = HKUnit(from: "count/min")
            let theRESThr = data.quantity.doubleValue(for: unit)
            print("theRESThr from query: \(theRESThr) BPM")
            
            self.restingHeartRate = theRESThr //give theHR whatever updated HR is...put in theHR bc we know the type of var for sure
            
//            let timeFormator = DateFormatter() //used to print just time next to previous heart rates
//            timeFormator.dateFormat = "hh:mm:ss"
//            self.timeDate = timeFormator.string(from: data.startDate) //hold time of queries
//            print("restHR Time date: \(self.timeDate)")
            let dateFormator = DateFormatter()
            dateFormator.dateFormat = "hh:mm:ss dd/MM/yyyy"
            let RestDate = dateFormator.string(from: data.startDate)
            self.theRestingString = RestDate
            
        }//end query
        
        healthStore.execute(query)
        
    } //end getRestHR()
    
    
    
//    let heartRateQuantity = HKUnit(from: "count/min")
//    @Published var hrCurrent : Int = 0
//
//    func hrQuery(quantityTypeIdentifier: HKQuantityTypeIdentifier) {
//        print("here on hrQuery")
//        // 1
//        let devicePredicate = HKQuery.predicateForObjects(from: [HKDevice.local()]) //say we want to grab hr from phone? unsure how often updates
//
//        // 2
//        //query returns changes to the HealthKit store... includes snapshot of new changes and continuous monitoring as a long running query
//        let updateHandler: (HKAnchoredObjectQuery, [HKSample]?, [HKDeletedObject]?, HKQueryAnchor?, Error?) -> Void = {
//            query, samples, deletedObjects, queryAnchor, error in
//
//            // 3
//            guard let samples = samples as? [HKQuantitySample] else { //return query results as HKQuantitySample in sample
//                return
//            }//end update handler
//
//            self.process(samples, type: quantityTypeIdentifier)
//
//        }//end update handler
//
//        // 4
//        let query = HKAnchoredObjectQuery(type: HKObjectType.quantityType(forIdentifier: quantityTypeIdentifier)!, predicate: devicePredicate, anchor: nil, limit: HKObjectQueryNoLimit, resultsHandler: updateHandler)
//
//        query.updateHandler = updateHandler
//
//        // 5
//
//        healthStore.execute(query)
//    }//end hrQuery
//
//    private func process(_ samples: [HKQuantitySample], type: HKQuantityTypeIdentifier) {
//            var lastHeartRate = 0.0
//
//            for sample in samples {
//                if type == .heartRate {
//                    lastHeartRate = sample.quantity.doubleValue(for: heartRateQuantity)
//                }
//
//                self.hrCurrent = Int(lastHeartRate)
//
//            }
//        print("hrCurrent: \(hrCurrent)")
//    }//end func process...failed query attempt
    
}//end workout manager

